#!/usr/bin/env python 

# TraceOverFlexi, v. 1.0 Python-fu GIMP plugin.
# April 2013, under Gnu Public License
# Script by Michael B. Stevens
# Place in your .gimp folder's plug-ins folder and make executable.
# Tested in Gimp 2.8.
# Will appear under the "Layer" menu, which seems vaguely more
#   appropriate than the "Filters" menu.  
# You must be drawing in black and white in RGB;
#   the original drawing must also be so. 
# Each run changes the previous drawing layer, so keep a backup
#    of your original drawing. 
  
from gimpfu import *

def colorLines(timg, tdrw, coltup, xtralayer, xtraopacity) :
    # Convert black lines to blue
    pdb.plug_in_exchange (timg,tdrw,0,0,0,  
        coltup[0],coltup[1],coltup[2],  
        254,254,254)

    if xtralayer:
        #First a new simi-opaque layer...
        blur = pdb.gimp_layer_new(timg, tdrw.width, 
            tdrw.height, RGB_IMAGE, "Simi-opaque", xtraopacity, 
            NORMAL_MODE) 
        pdb.gimp_context_set_background((255, 255, 255))
        pdb.gimp_drawable_fill(blur, BACKGROUND_FILL)
        pdb.gimp_image_insert_layer(timg, blur, None, -1)
    
    # Now make new layer to draw on...
    pdb.gimp_context_set_foreground((0, 0, 0))
    pdb.gimp_context_set_background((255, 255, 255))
    blr2 = pdb.gimp_layer_new(timg, tdrw.width, 
        tdrw.height, RGB_IMAGE, "Trace-over", 100, 
        MULTIPLY_MODE)
    pdb.gimp_drawable_fill(blr2, BACKGROUND_FILL)        
    pdb.gimp_image_insert_layer(timg, blr2, None, -1)  
    return 

register(
    "TraceOverFlexi",
    "Flexible tracing and corrections over rough drawing",
    "Flexible tracing and corrections over rough drawing",
    "Michael B. Stevens",
    "Michael B. Stevens",
    "March2013",
    "<Image>/Layer/TraceOverFlexi",
    "*", 
    [
        (PF_COLOR, "coltup", "Color", (80,80,80)),
        (PF_TOGGLE, "xtralayer", "Extra Layer", 0),
        (PF_SLIDER, "xtraopacity", "Extra Layer Opacity", 50, (0,100,1))
    ], 
    [], 
    colorLines
    )
     
main()

  
