#!/usr/bin/env python

#   Gimp-Python - allows the writing of Gimp plugins in Python.
#   Copyright (C) 1997  James Henstridge <james@daa.com.au>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Modified 2013-december by j.f.garcia www.arakne.es

from gimpfu import *
import pango

t = gettext.translation('gimp20-python', gimp.locale_directory, fallback=True)
_ = t.ugettext

PROC_NAME = 'python-fu-console2'

RESPONSE_BROWSE, RESPONSE_CLEAR, RESPONSE_SAVE = range(3)

def alert(msg):
  dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, msg)
  dlg.run()
  dlg.hide()

def do_console():
    import pygtk
    pygtk.require('2.0')

    import sys, gobject, gtk, gimpenums, gimpshelf, gimpui, pyconsole2

    namespace = {'__builtins__': __builtins__, '__name__': '__main__', '__doc__': None, 'gimp': gimp, 'pdb': gimp.pdb, 'shelf': gimpshelf.shelf}

    for s in gimpenums.__dict__.keys():
        if s[0] != '_':
            namespace[s] = getattr(gimpenums, s)

    class GimpConsole(pyconsole2.Console2):
        def __init__(self, quit_func=None):
            banner = ('GIMP %s Python Console\nPython %s\n' % (gimp.pdb.gimp_version(), sys.version))
            pyconsole2.Console2.__init__(self, locals=namespace, banner=banner, quit_func=quit_func)
        def _commit(self):
            pyconsole2.Console2._commit(self)
            gimp.displays_flush()

    class ConsoleDialog(gimpui.Dialog):
        def __init__(self):
            gimpui.Dialog.__init__(self, title=_("Python Console"),
                role=PROC_NAME, help_id=PROC_NAME,
                buttons=(gtk.STOCK_SAVE, RESPONSE_SAVE, gtk.STOCK_CLEAR, RESPONSE_CLEAR, _("_Browse..."), RESPONSE_BROWSE, gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
            self.set_alternative_button_order((gtk.RESPONSE_CLOSE, RESPONSE_BROWSE, RESPONSE_CLEAR, RESPONSE_SAVE))
            self.cons = GimpConsole(quit_func=lambda: gtk.main_quit())
            self.connect('response', self.response)
            self.browse_dlg = None
            self.save_dlg = None

            table = gtk.Table(6, 6, True)
            table.set_homogeneous(True)
            table.set_row_spacings(2)
            table.set_col_spacings(2)
            table.show()

            lb = gtk.Label('Inspect object:')
            lb.show()

        # create a TreeStore with one string column to use as the model
            self.treestore = gtk.TreeStore(str,str)
            for parent in dir(gtk):
                piter = self.treestore.append(None, ['%s' % parent,''])
            self.treeview = gtk.TreeView(self.treestore)
            self.tvcolumn = gtk.TreeViewColumn('Inspecting gimp')
            self.tvcol2 = gtk.TreeViewColumn('Type')
            self.treeview.append_column(self.tvcolumn)
            self.treeview.append_column(self.tvcol2)
            self.cell = gtk.CellRendererText()
            self.tvcolumn.pack_start(self.cell, True)
            self.tvcolumn.add_attribute(self.cell, 'text', 0)
            self.tvcol2.pack_start(self.cell, True)
            self.tvcol2.add_attribute(self.cell, 'text', 1)
            self.treeview.set_search_column(0)
            self.tvcolumn.set_sort_column_id(0)
            self.treeview.set_reorderable(True)

            self.tree_sel = self.treeview.get_selection()
            self.tree_sel.set_mode(gtk.SELECTION_SINGLE)
            self.tree_sel.connect("changed", self.onSelectionChanged)

            self.treeview.connect("row-activated", self.actRow)

            self.entry = gtk.Entry()
            self.entry.set_text('gimp')
            self.entry.show()
            self.entry.connect('activate',self.inspectclick)

            btn = gtk.Button('Inspect')
            btn.show()

            btn.connect("clicked", self.inspectclick)
            scrl = gtk.ScrolledWindow()
            scrl.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
            scrl.add(self.treeview)
            
            table.attach(lb         , 4, 5, 0, 1)
            table.attach(self.entry , 5, 6, 0, 1)
            table.attach(btn        , 4, 6, 1, 2)
            table.attach(scrl       , 4, 6, 2, 11)

            scrl2 = gtk.ScrolledWindow()
            scrl2.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
            self.txDoc = gtk.TextView(buffer=None)
            tb = self.txDoc.get_buffer()
            self.h_tag = tb.create_tag("h", size_points=11, weight=pango.WEIGHT_BOLD, background='#cccccc') 
            self.val_tag = tb.create_tag("v", foreground="#00F000", family='Monospace')
            self.txDoc.set_wrap_mode(gtk.WRAP_WORD)

            scrl2.add(self.txDoc)
            table.attach(scrl2       , 4, 6, 11, 14)

            vbox = gtk.VBox(False, 12)
            vbox.set_border_width(6)
            self.vbox.pack_start(vbox)

            vbox.pack_start(table)
            self.docBuff = ''
            scrl_win = gtk.ScrolledWindow()
            scrl_win.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)

            table.attach(scrl_win       , 0, 4, 0, 14)

            scrl_win.add(self.cons)
            self.set_default_size(500, 500)

        def onSelectionChanged(self, tree_selection):
            vals={'v':'','doc':''}
            tx = self.entry.get_text()
            self.cons.do_command(tx)
            #tmp = eval(tx, self.cons.locals)
            (model, iter) = tree_selection.get_selected()
            value = model.get_value(iter,0)
            tipo = model.get_value(iter,1)
            txObj = tx+'.'+value
            if value+'' != 'clean_all':
                tmp=eval(txObj, self.cons.locals)
            else:
                tmp=None
            tb = self.txDoc.get_buffer()
            ini,end = tb.get_bounds()
            tb.delete(ini,end)
            if tipo=='str' or tipo=='tuple':
                vals['v'] = tmp
            if hasattr(tmp,'__doc__'):
                if tmp.__doc__!=None:
                    vals['doc']=tmp.__doc__
            if ((vals['v']=='') and (vals['doc']=='')):
                tb.set_text(self.docBuff+'\n')
                p = tb.get_end_iter()
                tb.insert_with_tags( p, txObj + "\n", self.h_tag)
                p = tb.get_end_iter()
                tb.insert(p, tipo + '\n')
            else:
                p = tb.get_end_iter()
                tb.insert_with_tags( p, txObj + "\n", self.h_tag)
                p = tb.get_end_iter()
                tb.insert(p, tipo)
                if (vals['v']!=''):
                    p = tb.get_end_iter()
                    tb.insert(p, '=')
                    p = tb.get_end_iter()
                    tb.insert_with_tags(p, str(vals['v']),  self.val_tag)
                if (vals['doc']!=''):
                    p = tb.get_end_iter()
                    tb.insert(p, '\n' + vals['doc']+'\n')


        def inspectObj(self, tx):
            self.treestore.clear()
            self.cons.do_command(tx)
            tmp = eval(tx, self.cons.locals)
            tbuffer = self.txDoc.get_buffer()
            self.docBuff = ''
            if hasattr(tmp,'__doc__'):
                if tmp.__doc__!=None:
                    self.docBuff=tmp.__doc__ + ''
            tbuffer.set_text(self.docBuff)
            self.tvcolumn.set_title('Inspecting '+tx)
            all=dir(tmp)
            for ii in all:
                if hasattr(tmp,ii):
                    piter = self.treestore.append(None, [ii, ''])
            for n in self.treestore:
                v=self.treestore.get_value(n.iter,0)
                tmp=eval(tx + '.' + v, self.cons.locals)
                self.treestore.set_value(n.iter,1,type(tmp).__name__)
            # for ii in all:
                # gimp.message(ii+ ' =clean_all' +str(ii != 'clean_all'))
                # if ii != 'clean_all':
                    # tmp=eval(tx + '.' + ii, self.cons.locals)
                    # piter = self.treestore.append(None, [ii, type(tmp).__name__])
                # else:
                    # piter = self.treestore.append(None, ['clean_all', ''])

        def inspectclick(self, widget):
            tx = self.entry.get_text()
            self.inspectObj(tx)

        def actRow(self, treeview, path, column):
           model = treeview.get_model()
           iter = model.get_iter(path)
           buffer = self.cons.buffer
           buffer.insert_at_cursor(self.entry.get_text()+'.'+model.get_value(iter, 0))
           return

        def response(self, dialog, response_id):
            if response_id == RESPONSE_BROWSE:
                self.browse()
            elif response_id == RESPONSE_CLEAR:
                self.cons.banner = None
                self.cons.clear()
            elif response_id == RESPONSE_SAVE:
                self.save_dialog()
            else:
                gtk.main_quit()
            self.cons.grab_focus()

        def browse_response(self, dlg, response_id):
            if response_id != gtk.RESPONSE_APPLY:
                dlg.hide()
                return
            proc_name = dlg.get_selected()
            if not proc_name:
                return
            proc = pdb[proc_name]
            cmd = ''
            if len(proc.return_vals) > 0:
                cmd = ', '.join([x[1].replace('-', '_') for x in proc.return_vals]) + ' = '
            cmd = cmd + 'pdb.%s' % proc.proc_name.replace('-', '_')
            if len(proc.params) > 0 and proc.params[0][1] == 'run-mode':
                params = proc.params[1:]
            else:
                params = proc.params
            cmd = cmd + '(%s)' % ', '.join([x[1].replace('-', '_') for x in params])
            buffer = self.cons.buffer
            lines = buffer.get_line_count()
            iter = buffer.get_iter_at_line_offset(lines - 1, 4)
            buffer.delete(iter, buffer.get_end_iter())
            buffer.place_cursor(buffer.get_end_iter())
            buffer.insert_at_cursor(cmd)

        def browse(self):
            if not self.browse_dlg:
                dlg = gimpui.ProcBrowserDialog(_("Python Procedure Browser"), role=PROC_NAME, buttons=(gtk.STOCK_APPLY, gtk.RESPONSE_APPLY, gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
                dlg.set_default_response(gtk.RESPONSE_APPLY)
                dlg.set_alternative_button_order((gtk.RESPONSE_CLOSE, gtk.RESPONSE_APPLY))
                dlg.connect('response', self.browse_response)
                dlg.connect('row-activated', lambda dlg: dlg.response(gtk.RESPONSE_APPLY))
                self.browse_dlg = dlg
            self.browse_dlg.present()

        def save_response(self, dlg, response_id):
            if response_id == gtk.RESPONSE_DELETE_EVENT:
                self.save_dlg = None
                return
            elif response_id == gtk.RESPONSE_OK:
                filename = dlg.get_filename()
                try:
                    logfile = open(filename, 'w')
                except IOError, e:
                    gimp.message(_("Could not open '%s' for writing: %s") % (filename, e.strerror))
                    return
                buffer = self.cons.buffer
                start = buffer.get_start_iter()
                end = buffer.get_end_iter()
                log = buffer.get_text(start, end, False)
                try:
                    logfile.write(log)
                    logfile.close()
                except IOError, e:
                    gimp.message(_("Could not write to '%s': %s") % (filename, e.strerror))
                    return
            dlg.hide()

        def save_dialog(self):
            if not self.save_dlg:
                dlg = gtk.FileChooserDialog(_("Save Python-Fu Console Output"), parent=self, action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
                dlg.set_default_response(gtk.RESPONSE_OK)
                dlg.set_alternative_button_order((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))
                dlg.connect('response', self.save_response)
                self.save_dlg = dlg
            self.save_dlg.present()

        def run(self):
            self.show_all()
            gtk.main()

    ConsoleDialog().run()

register( PROC_NAME, N_("Interactive GIMP Python interpreter"), "Type in commands and see results", "James Henstridge", "James Henstridge", "1997-1999", N_("_Console extended"), "", [], [], do_console, menu="<Image>/Filters/Languages/Python-Fu", domain=("gimp20-python", gimp.locale_directory))

main()
# 207